@@ -1,9 +1,9 @@
 from typing import Any, List
 
-from aqueduct.aqueduct_client import Client, get_apikey, infer_requirements
+from aqueduct.client import Client, get_apikey, global_config, infer_requirements
 from aqueduct.constants import exports
+from aqueduct.constants.enums import CheckSeverity, LoadUpdateMode
 from aqueduct.decorator import check, metric, op, to_operator
-from aqueduct.enums import CheckSeverity, LoadUpdateMode
 from aqueduct.flow import Flow
 from aqueduct.schedule import DayOfMonth, DayOfWeek, Hour, Minute, daily, hourly, monthly, weekly
 
